#ifndef CUDA_AES_CUHPP_
#define CUDA_AES_CUHPP_

// load tables to shared memory from global memory
#define TOTAL_WORDS_ENCRYPTION_TABLES 256*4
#define __SHARED_ENCRYPTION_TABLES__() \
	__shared__ uint32_t sT0[256]; \
	__shared__ uint32_t sT1[256]; \
	__shared__ uint32_t sT2[256]; \
	__shared__ uint32_t sT3[256]; \
  	uint32_t* T0_32bits = (uint32_t*) T0; \
  	uint32_t* T1_32bits = (uint32_t*) T1; \
 	uint32_t* T2_32bits = (uint32_t*) T2; \
 	uint32_t* T3_32bits = (uint32_t*) T3; \
	int n_threads_loading; \
	if(blockDim.x < TOTAL_WORDS_ENCRYPTION_TABLES) { \
		n_threads_loading = blockDim.x; \
	} else { \
		n_threads_loading = TOTAL_WORDS_ENCRYPTION_TABLES; \
	} \
	if(threadIdx.x < n_threads_loading) { \
		int loadsPerThread = TOTAL_WORDS_ENCRYPTION_TABLES / n_threads_loading; \
		int beginWord = loadsPerThread*threadIdx.x; \
		__LOG_TRACE__("my %d loads begin at %d ",loadsPerThread,beginWord); \
		for(int i = 0; i < loadsPerThread; i++) { \
			int wi = i+beginWord; \
			if(wi < 256) { \
				__LOG_TRACE__("%d storing T0[%d] in shared memory",wi,wi); \
				sT0[wi] = T0_32bits[wi]; \
			} else if(wi < 256*2) { \
				__LOG_TRACE__("%d storing T1[%d] in shared memory",wi,wi%256); \
				sT1[wi%256] = T1_32bits[wi%256]; \
			} else if(wi < 256*3) { \
				__LOG_TRACE__("%d storing T2[%d] in shared memory",wi,wi%256); \
				sT2[wi%256] = T2_32bits[wi%256]; \
			} else if(wi < 256*4) { \
				__LOG_TRACE__("%d storing T3[%d] in shared memory",wi,wi%256); \
				sT3[wi%256] = T3_32bits[wi%256]; \
			} \
		} \
	} \
	__syncthreads();

#define TOTAL_WORDS_DECRYPTION_TABLES 256*4+64
#define __SHARED_DECRYPTION_TABLES__() \
	__shared__ uint32_t sT0[256]; \
	__shared__ uint32_t sT1[256]; \
	__shared__ uint32_t sT2[256]; \
	__shared__ uint32_t sT3[256]; \
	__shared__ uint32_t sT4[64]; \
 	uint32_t* T0_32bits = (uint32_t*) T0; \
  	uint32_t* T1_32bits = (uint32_t*) T1; \
 	uint32_t* T2_32bits = (uint32_t*) T2; \
 	uint32_t* T3_32bits = (uint32_t*) T3; \
 	uint32_t* T4_32bits = (uint32_t*) T4; \
	int n_threads_loading; \
	if(blockDim.x < TOTAL_WORDS_DECRYPTION_TABLES) { \
		n_threads_loading = blockDim.x; \
	} else { \
		n_threads_loading = TOTAL_WORDS_DECRYPTION_TABLES; \
	} \
	if(threadIdx.x < n_threads_loading) { \
		int loadsPerThread = (TOTAL_WORDS_DECRYPTION_TABLES) / n_threads_loading; \
		int beginWord = loadsPerThread*threadIdx.x; \
		__LOG_TRACE__("my %d loads begin at %d (n_thrads_loading => %d)",loadsPerThread,beginWord,n_threads_loading); \
		for(int i = 0; i < loadsPerThread; i++) { \
			int wi = i+beginWord; \
			if(wi < 256) { \
				__LOG_TRACE__("%d storing T0[%d] in shared memory",wi,wi); \
				sT0[wi] = T0_32bits[wi]; \
			} else if(wi < 256*2) { \
				__LOG_TRACE__("%d storing T1[%d] in shared memory",wi,wi%256); \
				sT1[wi%256] = T1_32bits[wi%256]; \
			} else if(wi < 256*3) { \
				__LOG_TRACE__("%d storing T2[%d] in shared memory",wi,wi%256); \
				sT2[wi%256] = T2_32bits[wi%256]; \
			} else if(wi < 256*4) { \
				__LOG_TRACE__("%d storing T3[%d] in shared memory",wi,wi%256); \
				sT3[wi%256] = T3_32bits[wi%256]; \
			} else if(wi < 256*4+64) { \
				__LOG_TRACE__("%d storing T4[%d] in shared memory",wi,wi%64); \
				sT4[wi%64] = T4_32bits[wi%64]; \
			} \
		} \
	} \
	__syncthreads();

	// We don't need to check additional because Paracrypt
	//  always use a pair number of threads per thread-block
	//int additional = TOTAL_WORDS % n_threads_loading;
	//int additionalIndex = (TOTAL_WORDS-additional)+threadIdx.x;

	/*
	if(additionalIndex < additional) {
		if(additionalIndex < 256) {
			__LOG_TRACE__("%d storing T0[%d] in shared memory",additionalIndex,additionalIndex);
			sT0[additionalIndex] = T0[additionalIndex];
		} else if(additionalIndex < 256*2) {
			__LOG_TRACE__("%d storing T1[%d] in shared memory",additionalIndex,additionalIndex%256);
			sT1[additionalIndex] = T1[additionalIndex];
		} else if(additionalIndex < 256*3) {
			__LOG_TRACE__("%d storing T2[%d] in shared memory",additionalIndex,additionalIndex%256);
			sT2[additionalIndex%256] = T2[additionalIndex%256];
		} else if(additionalIndex < 256*4) {
			__LOG_TRACE__("%d storing T3[%d] in shared memory",additionalIndex,additionalIndex%256);
			sT3[additionalIndex%256] = T3[additionalIndex%256];
		}
	}
	*/
/*
#if defined(DEBUG) && defined(DEVEL)
			if(threadIdx.x == 0) {
				int i;
				for(i = 0; i < 256; i++) {
					__LOG_TRACE__("sT0[%d] => 0x%04x",i,sT0[i]);
				}
				for(i = 0; i < 256; i++) {
					__LOG_TRACE__("sT1[%d] => 0x%04x",i,sT1[i]);
				}
				for(i = 0; i < 256; i++) {
					__LOG_TRACE__("sT2[%d] => 0x%04x",i,sT2[i]);
				}
				for(i = 0; i < 256; i++) {
					__LOG_TRACE__("sT3[%d] => 0x%04x",i,sT3[i]);
				}
			}
#endif
*/
#endif
